<?php
/**
* SEO4ALL. Extension for the phpBB Forum Software package.
*
* @copyright (c) 2017 https://thetopfew.com
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
**/

namespace thetopfew\seo4all\acp;

use thetopfew\seo4all\service\constants\prefixes;

class functions_module
{
	/** @var \phpbb\cache\service */
	protected $cache;

	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;

	/** @var \phpbb\log\log */
	protected $log;

	/** @var \phpbb\request\request */
	protected $request;

	/** @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\user */
	protected $user;

	/** @var string */
	protected $phpbb_root_path;
	
	/** @var string */
	protected $phpbb_admin_path;

	/** @var string */
	protected $php_ext;

	/** @var ContainerInterface */
	protected $phpbb_container;

	/** @var string */
	public $u_action;

	/** @var \thetopfew\seo4all\service\tags_manager */
	protected $tags_manager;
	
	
	public function __construct()
	{
		global $phpbb_container;
		$this->tags_manager = $phpbb_container->get('thetopfew.seo4all.tags_manager');
	}
	
	function main($id, $mode)
	{
		global $cache, $config, $db, $request, $template, $user, $phpbb_root_path, $phpbb_admin_path, $phpEx, $phpbb_container;
		
		$this->driver = $cache->get_driver();
		$this->config = $config;
		$this->config_text = $phpbb_container->get('config_text');
		$this->db = $db;
		$this->log = $phpbb_container->get('log');
		$this->request = $request;
		$this->template = $template;
		$this->user = $user;
		$this->phpbb_root_path = $phpbb_root_path;
		$this->phpbb_admin_path = $phpbb_admin_path;
		$this->php_ext = $phpEx;
		

		switch ($mode)
		{
			case 'settings':
				$this->tpl_name = 'acp_tag_settings';
				$this->page_title = 'ACP_TOPICTAGS_SETTINGS';
				$this->handle_settings();
			break;
			case 'whitelist':
				$this->tpl_name = 'acp_tag_whitelist';
				$this->page_title = 'ACP_TOPICTAGS_WHITELIST';
				$acp_whiteblacklist_controller = $phpbb_container->get('thetopfew.seo4all.acp.white_and_blacklist_controller');
				$acp_whiteblacklist_controller->manage_whitelist($this->u_action);
			break;
			case 'blacklist':
				$this->tpl_name = 'acp_tag_blacklist';
				$this->page_title = 'ACP_TOPICTAGS_BLACKLIST';
				$acp_whiteblacklist_controller = $phpbb_container->get('thetopfew.seo4all.acp.white_and_blacklist_controller');
				$acp_whiteblacklist_controller->manage_blacklist($this->u_action);
			break;
			case 'tags':
				$this->tpl_name = 'acp_tag_manager';
				$this->page_title = 'ACP_TOPICTAGS_MANAGE_TAGS';
				$acp_manage_tags_controller = $phpbb_container->get('thetopfew.seo4all.acp.manage_tags_controller');
				$acp_manage_tags_controller->manage_tags($mode, $this->u_action, $id);
			break;
			
			case 'sitemap':
			// no break
			default:
				$this->tpl_name = 'acp_sitemap';
				$this->page_title = 'ACP_SITEMAP_SETTINGS';
				add_form_key('acp_seo4all_sitemap');
				$action = $this->request->variable('action', '');
				$error = array();
		}
		
		$sitemap_url = generate_board_url() . '/sitemap.xml';

		if ($action == 'sync')
		{
			// Try to override limits
			@set_time_limit(0);
			$mem_limit = @ini_get('memory_limit');

			if (!empty($mem_limit))
			{
				$unit = strtolower(substr($mem_limit, -1, 1));
				$mem_limit = (int) $mem_limit;

				if ($unit == 'k')
				{
					$mem_limit = floor($mem_limit / 1024);
				}
				else if ($unit == 'g')
				{
					$mem_limit *= 1024;
				}
				else if (is_numeric($unit))
				{
					$mem_limit = floor((int) ($mem_limit . $unit) / 1048576);
				}

				$mem_limit = max(128, $mem_limit) . 'M';
			}
			else
			{
				$mem_limit = '128M';
			}

			@ini_set('memory_limit', $mem_limit);

			$start = $this->request->variable('start', 0);

			// Limit updated posts to a specified value
			$limit = 5000;

			// Do the sync
			$sql = 'SELECT post_id FROM ' . POSTS_TABLE . '
					ORDER BY post_id ASC';
			$result = $this->db->sql_query_limit($sql, $limit, $start);

			$post_ids = array();

			while ($row = $this->db->sql_fetchrow($result))
			{
				$post_ids[] = (int) $row['post_id'];
			}
			$this->db->sql_freeresult($result);

			if (sizeof($post_ids))
			{
				$upd_sql = 'UPDATE ' . POSTS_TABLE . ' SET post_modified =
					CASE
						WHEN post_time > post_edit_time THEN post_time
						ELSE post_edit_time
					END
					WHERE ' . $this->db->sql_in_set('post_id', $post_ids) . '
					AND post_modified = 0';
				$this->db->sql_query($upd_sql);
			}

			$sql = 'SELECT COUNT(post_id) AS postcount FROM ' . POSTS_TABLE;
			$result = $this->db->sql_query($sql, 3600);
			$count = (int) $this->db->sql_fetchfield('postcount');
			$this->db->sql_freeresult($result);

			if ($count > ($start + $limit))
			{
				$percent = round((($start + $limit) / $count) * 100, 2);
				$message = sprintf($this->user->lang['SEO4ALL_MAP_SYNC_PROCESS'], $percent, ($start + $limit), $count);

				$next = $start + $limit;
				meta_refresh(1, append_sid("{$phpbb_admin_path}index.$phpEx?i=-thetopfew-seo4all-acp-functions_module&amp;action=sync&amp;start=$next"));

				trigger_error($message . '<br />');
			}
			else
			{
				$this->driver->destroy('_sitemap_file');
				
				trigger_error(sprintf($this->user->lang['SEO4ALL_MAP_SYNC_COMPLETE'], append_sid("{$phpbb_admin_path}index.$phpEx?i=-thetopfew-seo4all-acp-functions_module&amp;mode=sitemap")));
			}
		}
		else
		{
			$sql = 'SELECT 1 FROM ' . POSTS_TABLE . '
				WHERE post_modified = 0';
			$this->db->sql_query_limit($sql, 1);

			if ($this->db->sql_affectedrows())
			{
				$error[] = sprintf($this->user->lang['SEO4ALL_MAP_SYNC_REQ'], append_sid("{$phpbb_admin_path}index.$phpEx?i=-thetopfew-seo4all-acp-functions_module&amp;action=sync"));
			}

			if ($this->request->is_set_post('submit'))
			{
				if (!check_form_key('acp_seo4all_sitemap'))
				{
					trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
				}

				$this->config->set('sitemap_cache', max(0, $this->request->variable('sitemap_cache', 0)));
				$this->config->set('sitemap_url_limit', min(50000, max(1, $this->request->variable('sitemap_url_limit', 0))));
				$this->config->set('sitemap_batch_size', $this->request->variable('sitemap_batch_size', $this->config['sitemap_batch_size']));
				$this->config_text->set('sitemap_excluded', implode(',', $this->request->variable('sitemap_excluded', array(0))));
				$this->config->set('sitemap_prior_0', min(1, max(0, $this->request->variable('sitemap_prior_0', $this->config['sitemap_prior_0']))));
				$this->config->set('sitemap_prior_1', min(1, max(0, $this->request->variable('sitemap_prior_1', $this->config['sitemap_prior_1']))));
				$this->config->set('sitemap_prior_2', min(1, max(0, $this->request->variable('sitemap_prior_2', $this->config['sitemap_prior_2']))));
				$this->config->set('sitemap_prior_3', min(1, max(0, $this->request->variable('sitemap_prior_3', $this->config['sitemap_prior_3']))));
				$this->config->set('sitemap_prior_4', min(1, max(0, $this->request->variable('sitemap_prior_4', $this->config['sitemap_prior_4']))));
				$this->config->set('sitemap_prior_f', min(1, max(0, $this->request->variable('sitemap_prior_f', $this->config['sitemap_prior_f']))));
				$this->config->set('sitemap_freq_0', $this->request->variable('sitemap_freq_0', $this->config['sitemap_freq_0']));
				$this->config->set('sitemap_freq_1', $this->request->variable('sitemap_freq_1', $this->config['sitemap_freq_1']));
				$this->config->set('sitemap_freq_2', $this->request->variable('sitemap_freq_2', $this->config['sitemap_freq_2']));
				$this->config->set('sitemap_freq_3', $this->request->variable('sitemap_freq_3', $this->config['sitemap_freq_3']));
				$this->config->set('sitemap_freq_4', $this->request->variable('sitemap_freq_4', $this->config['sitemap_freq_4']));
				$this->config->set('sitemap_freq_f', $this->request->variable('sitemap_freq_f', $this->config['sitemap_freq_f']));

				$this->driver->destroy('_sitemap_file');

				$this->log->add('admin', $this->user->data['user_id'], $this->user->ip, 'SEO4ALL_MAP_SETTINGS_UPDATED');
				trigger_error($this->user->lang['SEO4ALL_MAP_SAVED'] . adm_back_link($this->u_action));
			}
		}
		
		$this->template->assign_vars(array(
			'SEO4ALL_VERSION'					=> sprintf($this->user->lang['SEO4ALL_VERSION'], '<strong>' . $this->config['seo4all_version'] . '</strong>'),
			'SEO4ALL_MAP_CACHE_TIME'			=> $this->config['sitemap_cache'],
			'SEO4ALL_MAP_URL'					=> sprintf($this->user->lang['SEO4ALL_MAP_URL'], "{$phpbb_root_path}sitemap.xml", $sitemap_url),
			'SEO4ALL_MAP_URL_COUNT'				=> sprintf($this->user->lang['SEO4ALL_MAP_URL_COUNT'], '<strong>' . $this->config['sitemap_url_count'] . '</strong>'),
			'SEO4ALL_MAP_URL_LIMIT'				=> $this->config['sitemap_url_limit'],
			'SEO4ALL_MAP_BATCH_SIZE'			=> $this->config['sitemap_batch_size'],
			'SEO4ALL_MAP_EXCLUDED_FORUMS'		=> make_forum_select(explode(',', $this->config_text->get('sitemap_excluded')), false, false, true),
			'SEO4ALL_MAP_PRIORITY_0'			=> $this->config['sitemap_prior_0'],
			'SEO4ALL_MAP_PRIORITY_1'			=> $this->config['sitemap_prior_1'],
			'SEO4ALL_MAP_PRIORITY_2'			=> $this->config['sitemap_prior_2'],
			'SEO4ALL_MAP_PRIORITY_3'			=> $this->config['sitemap_prior_3'],
			'SEO4ALL_MAP_PRIORITY_4'			=> $this->config['sitemap_prior_4'],
			'SEO4ALL_MAP_PRIORITY_F'			=> $this->config['sitemap_prior_f'],
			'SEO4ALL_MAP_FREQ_0'				=> $this->config['sitemap_freq_0'],
			'SEO4ALL_MAP_FREQ_1'				=> $this->config['sitemap_freq_1'],
			'SEO4ALL_MAP_FREQ_2'				=> $this->config['sitemap_freq_2'],
			'SEO4ALL_MAP_FREQ_3'				=> $this->config['sitemap_freq_3'],
			'SEO4ALL_MAP_FREQ_4'				=> $this->config['sitemap_freq_4'],
			'SEO4ALL_MAP_FREQ_F'				=> $this->config['sitemap_freq_f'],

			//'MAP_ERROR_MSG'					=> implode('<br />', $error),
			'MAP_ERROR_MSG'						=> sprintf($this->user->lang['SEO4ALL_MAP_SYNC_REQ'], append_sid("{$phpbb_admin_path}index.$phpEx?i=-thetopfew-seo4all-acp-functions_module&amp;action=sync")),
			'S_ARTICLE'							=> defined('POST_ARTICLE'),
			'MAP_S_ERROR'						=> (sizeof($error)) ? true : false,
			'U_ACTION'							=> $this->u_action,
		));
	}
//end sitemap

	/**
	 * Default 'Tag Settings' tab
	 */
	private function handle_settings()
	{
		global $config, $request, $template, $user;
		// Define the name of the form for use as a form key
		$form_name = 'topictags';
		add_form_key($form_name);

		$errors = array();

		if ($request->is_set_post('submit'))
		{
			if (!check_form_key($form_name))
			{
				trigger_error('FORM_INVALID');
			}

			$commit_to_db = true;
			$msg = array();

			$regex = utf8_normalize_nfc($request->variable(prefixes::CONFIG . '_allowed_tags_regex', $user->lang('ACP_SEO4ALL_TAGS_REGEX_DEFAULT'), true));
			if (empty($regex))
			{
				$commit_to_db = false;
				$errors[] = $user->lang('ACP_SEO4ALL_TAGS_REGEX_EMPTY');
			}

			$exp_for_users = utf8_normalize_nfc($request->variable(prefixes::CONFIG . '_allowed_tags_exp_for_users', $user->lang('ACP_SEO4ALL_TAGS_REGEX_EXP_FOR_USERS_DEFAULT'), true));
			if (empty($exp_for_users))
			{
				$commit_to_db = false;
				$errors[] = $user->lang('ACP_SEO4ALL_TAGS_EXP_FOR_USERS_EMPTY');
			}

			if ($commit_to_db)
			{
				$config->set(prefixes::CONFIG . '_display_tags_in_viewforum', $request->variable(prefixes::CONFIG . '_display_tags_in_viewforum', 0));
				$config->set(prefixes::CONFIG . '_allowed_tags_regex', $regex);
				$config->set(prefixes::CONFIG . '_allowed_tags_exp_for_users', $exp_for_users);
				$config->set(prefixes::CONFIG . '_display_tagcloud_on_index', $request->variable(prefixes::CONFIG . '_display_tagcloud_on_index', 0));
				$config->set(prefixes::CONFIG . '_display_tagcount_in_tagcloud', $request->variable(prefixes::CONFIG . '_display_tagcount_in_tagcloud', 0));
				$max_tags_in_tagcloud = $request->variable(prefixes::CONFIG . '_max_tags_in_tagcloud', 20);
				if ($max_tags_in_tagcloud < 0)
				{
					$max_tags_in_tagcloud = 0;
				}
				$config->set(prefixes::CONFIG . '_max_tags_in_tagcloud', $max_tags_in_tagcloud);
				$config->set(prefixes::CONFIG . '_convert_space_to_minus', $request->variable(prefixes::CONFIG . '_convert_space_to_minus', 1));

				$deleted_assignments_count = 0;
				$delete_unused_tags = false;

				if ($request->variable(prefixes::CONFIG . '_enable_in_all_forums', 0) > 0)
				{
					$count_affected = $this->tags_manager->enable_tags_in_all_forums();
					$msg[] = $user->lang('TOPICTAGS_ENABLE_IN_ALL_FORUMS_DONE', $count_affected);
				}

				if ($request->variable(prefixes::CONFIG . '_disable_in_all_forums', 0) > 0)
				{
					$count_affected = $this->tags_manager->disable_tags_in_all_forums();
					$msg[] = $user->lang('TOPICTAGS_DISABLE_IN_ALL_FORUMS_DONE', $count_affected);
				}

				if ($request->variable(prefixes::CONFIG . '_calc_count_tags', 0) > 0)
				{
					$this->tags_manager->calc_count_tags();
					$msg[] = $user->lang('TOPICTAGS_CALC_COUNT_TAGS_DONE');
				}

				if ($request->variable(prefixes::CONFIG . '_prune', 0) > 0)
				{
					$deleted_assignments_count += $this->tags_manager->delete_assignments_where_topic_does_not_exist();
					$delete_unused_tags = true;
				}

				if ($request->variable(prefixes::CONFIG . '_prune_forums', 0) > 0)
				{
					$deleted_assignments_count += $this->tags_manager->delete_tags_from_tagdisabled_forums();
					$delete_unused_tags = true;
				}

				if ($request->variable(prefixes::CONFIG . '_prune_invalid_tags', 0) > 0)
				{
					$deleted_assignments_count += $this->tags_manager->delete_assignments_of_invalid_tags();
					$delete_unused_tags = true;
				}

				if ($delete_unused_tags)
				{
					$deleted_tags_count = $this->tags_manager->delete_unused_tags();
					$msg[] = $user->lang('TOPICTAGS_PRUNE_TAGS_DONE', $deleted_tags_count);
				}

				if ($deleted_assignments_count > 0)
				{
					$msg[] = $user->lang('TOPICTAGS_PRUNE_ASSIGNMENTS_DONE', $deleted_assignments_count);
				}

				if (empty($msg))
				{
					$msg[] = $user->lang('TOPICTAGS_SETTINGS_SAVED');
				}
				trigger_error(join('<br/>', $msg) . adm_back_link($this->u_action));
			}
		}
		$all_enabled = $this->tags_manager->is_enabled_in_all_forums();
		$all_disabled = ($all_enabled ? false : $this->tags_manager->is_disabled_in_all_forums());
		$template->assign_vars(array(
			'TOPICTAGS_DISPLAY_TAGS_IN_VIEWFORUM'    => $config[prefixes::CONFIG . '_display_tags_in_viewforum'],
			'TOPICTAGS_DISPLAY_TAGCLOUD_ON_INDEX'    => $config[prefixes::CONFIG . '_display_tagcloud_on_index'],
			'TOPICTAGS_DISPLAY_TAGCOUNT_IN_TAGCLOUD' => $config[prefixes::CONFIG . '_display_tagcount_in_tagcloud'],
			'TOPICTAGS_MAX_TAGS_IN_TAGCLOUD'         => $config[prefixes::CONFIG . '_max_tags_in_tagcloud'],
			'TOPICTAGS_ALLOWED_TAGS_REGEX'           => $config[prefixes::CONFIG . '_allowed_tags_regex'],
			'TOPICTAGS_ALLOWED_TAGS_EXP_FOR_USERS'   => $config[prefixes::CONFIG . '_allowed_tags_exp_for_users'],
			'TOPICTAGS_CONVERT_SPACE_TO_MINUS'       => $config[prefixes::CONFIG . '_convert_space_to_minus'],
			'TOPICTAGS_IS_ENABLED_IN_ALL_FORUMS'     => $all_enabled,
			'TOPICTAGS_IS_DISABLED_IN_ALL_FORUMS'    => $all_disabled,
			'S_ERROR'                                => (sizeof($errors)) ? true : false,
			'ERROR_MSG'                              => implode('<br />', $errors),
			'U_ACTION'                               => $this->u_action,
		));
	}
}
