<?php
/**
* SEO4ALL. Extension for the phpBB Forum Software package.
*
* @copyright (c) 2017 https://thetopfew.com
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
**/

namespace thetopfew\seo4all\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
* Event listener
*/
class acp_listener implements EventSubscriberInterface
{
	/** @var \phpbb\request\request_interface */
	private $request;

	/** @var \phpbb\user */
	private $user;

	/** @var \thetopfew\seo4all\service\tags_manager */
	private $tags_manager;

	/**
	 * Constructor
	 */
	public function __construct(
			\phpbb\request\request_interface $request,
			\phpbb\user $user,
			\thetopfew\seo4all\service\tags_manager $tags_manager)
	{
		$this->request = $request;
		$this->user = $user;
		$this->tags_manager = $tags_manager;
	}

	/**
	* Assign functions defined in this class to event listeners in the core
	*/
	public static function getSubscribedEvents()
	{
		return array(
			'core.acp_manage_forums_display_form'      => 'acp_manage_forums_display_form',
			'core.acp_manage_forums_validate_data'     => 'acp_manage_forums_validate_data',
			'core.acp_manage_forums_update_data_after' => 'acp_manage_forums_update_data_after',
		);
	}

	public function acp_manage_forums_display_form($event)
	{
		$data = $event->get_data();

		$data['template_data']['S_SEO4ALL_TAGS_ENABLED'] = $data['forum_data']['seo_topictags_enabled'];
		$prune = $this->request->variable('seo4all_tags_prune', 0);
		$data['template_data']['S_SEO4ALL_TAGS_PRUNE'] = $prune;

		$event->set_data($data);
	}

	public function acp_manage_forums_validate_data($event)
	{
		$data = $event->get_data();

		$status = $this->request->variable('seo_topictags_enabled', 0);
		// ensure 0 or 1
		$status = ($status ? 1 : 0);
		$data['forum_data']['seo_topictags_enabled'] = $status;

		// pruning requires the tagging to be disabled for this forum to prevent accidental deletion of tags
		$prune = $this->request->variable('seo4all_tags_prune', 0);
		if ($prune && $status)
		{
			$data['errors'][] = $this->user->lang('ACP_SEO4ALL_TAGS_PRUNING_REQUIRES_TAGGING_DISABLED');
		}

		$event->set_data($data);
	}

	public function acp_manage_forums_update_data_after($event)
	{
		$status = $this->request->variable('seo_topictags_enabled', 0);
		$prune = $this->request->variable('seo4all_tags_prune', 0);
		if (!$status && $prune)
		{
			$data = $event->get_data();
			$forum_id = (int) $data['forum_data']['forum_id'];
			$this->tags_manager->delete_tags_from_tagdisabled_forums(array($forum_id));
			$this->tags_manager->delete_unused_tags();
		}
		$this->tags_manager->calc_count_tags();
	}
}