<?php
/**
* SEO4ALL. Extension for the phpBB Forum Software package.
*
* @copyright (c) 2017 https://thetopfew.com
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
**/

namespace thetopfew\seo4all\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use thetopfew\seo4all\service\constants\permissions;
use thetopfew\seo4all\service\constants\prefixes;
use thetopfew\seo4all\service\constants\tables;

/**
* Event listener
*/
class listener implements EventSubscriberInterface
{
	const MAX_LENGTH = 158;
	
	/** @var \phpbb\auth\auth */
	protected $auth;
	
	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;
	
	/** @var \phpbb\controller\helper */
	protected $helper;

	/** @var \phpbb\language\language */
	protected $language;
	
	/** @var \phpbb\request\request */
	protected $request;

	/** @var \thetopfew\seo4all\service\tagcloud_manager */
	protected $tagcloud_manager;
	
	/** @var \thetopfew\seo4all\service\tags_manager */
	protected $tags_manager;
	
	/** @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\user */
	protected $user;

	/** @var string phpBB root path */
	protected $phpbb_root_path;

	/** @var string phpEx */
	protected $php_ext;
	
	/**
	* Constructor
	*/
	public function __construct(
			\phpbb\auth\auth $auth,
			\phpbb\config\config $config,
			\phpbb\db\driver\driver_interface $db,
			\phpbb\controller\helper $helper,
			\phpbb\language\language $language,
			\phpbb\request\request $request,
			\thetopfew\seo4all\service\tagcloud_manager $tagcloud_manager,
			\thetopfew\seo4all\service\tags_manager $tags_manager,
			\phpbb\template\template $template,
			\phpbb\user $user,
			$phpbb_root_path,
			$php_ext)
	{
		$this->auth = $auth;
		$this->config = $config;
		$this->db = $db;
		$this->helper = $helper;
		$this->language = $language;
		$this->request = $request;
		$this->tagcloud_manager = $tagcloud_manager;
		$this->tags_manager = $tags_manager;
		$this->template = $template;
		$this->user = $user;
		$this->phpbb_root_path = $phpbb_root_path;
		$this->php_ext = $php_ext;
	}
	
	/**
	* Assign functions defined in this class to event listeners in the core
	*/
	static public function getSubscribedEvents()
	{
		return array(
			'core.user_setup'   							=> 'seo4all_startup',
			// USER TOPIC DESCRIPTION
			'core.permissions'								=> 'add_permissions',
			'core.posting_modify_template_vars'				=> 'modify_topic_meta_data',
			'core.posting_modify_submit_post_before'		=> 'topic_desc_add',
			'core.posting_modify_message_text'				=> 'modify_message_text',
			'core.submit_post_modify_sql_data'				=> 'submit_post_modify_sql_data',
			'core.viewtopic_modify_page_title'				=> 'topic_desc_add_viewtopic',
			'core.viewforum_modify_topicrow'				=> 'modify_topicrow',
			'core.search_modify_tpl_ary'					=> 'search_modify_tpl_ary',
			'core.mcp_view_forum_modify_topicrow'			=> 'modify_topicrow',
			// AUTO TOPIC DESCRIPTION
			'core.display_forums_modify_forum_rows'			=> 'forumlist_description',
			'core.viewforum_get_topic_data'					=> 'viewforum_description',
			'core.viewtopic_assign_template_vars_before'    => 'viewtopic_assign_template_vars_before',
			// TOPIC TAGS
			'core.index_modify_page_title'                  => 'index_modify_page_title',
			'core.modify_posting_parameters'                => 'modify_posting_parameters',
			'thetopfew.seo4all.viewforum_modify_topicrow' 	=> 'modify_topicrow',
			'core.delete_topics_before_query'               => 'delete_topics_before_query',
			// UPDATE SITEMAP
			'core.submit_post_end'							=> 'update_post_modified_time',
			'core.page_footer_after'						=> 'seo_return',
		);
	}
	
	public function seo4all_startup($event) 
	{
		$this->language->add_lang('common', 'thetopfew/seo4all');
	}
   
	/**
	* Add administrative permissions to manage forums
	*
	* @param object $event The event object
	* @return null
	* @access public
	*/
	public function add_permissions($event)
	{
		$permissions = $event['permissions'];
		$permissions[permissions::USE_T_DESC] = array(permissions::USE_T_DESC, 'cat' => 'post');
		$permissions[permissions::USE_TAGS] = array(permissions::USE_TAGS, 'cat' => 'post');
		$event['permissions'] = $permissions;
	}
	
	/** 
	 * Event: core.posting_modify_template_vars
	 *
	 * Send the description and tags on edits or preview to the template
	 *
	 * @param $event
	 */
	public function modify_topic_meta_data($event)
	{
		$forum_id = $event['forum_id'];

		$mode = $event['mode'];
		$post_data = $event['post_data'];
		$page_data = $event['page_data'];
		if ($this->auth->acl_get(permissions::USE_T_DESC, $forum_id) && ($mode == 'post' || ($mode == 'edit' && $post_data['topic_first_post_id'] == $post_data['post_id'])))
		{
			// we need to set the variable upon posting only...only forum table is queried
			if ($mode == 'post')
			{
				$post_data[tables::TOPICDESC] = '';
			}
			$page_data['TOPIC_DESC'] = $this->request->variable('topic_desc', $post_data[tables::TOPICDESC], true);
			$page_data['S_DESC_TOPIC'] = true;
		}
		
		$event['post_data']	= $post_data;
		$event['page_data']	= $page_data;
		
		
		if ($this->auth->acl_gets(permissions::USE_TAGS, permissions::ADMIN_EDIT_TAGS, permissions::MOD_EDIT_TAGS))
		{
			$data = $event->get_data();
			$forum_id = $data['forum_id'];

			if (!$this->tags_manager->is_tagging_enabled_in_forum($forum_id))
			{
				return;
			}

			$topic_id = false;

			if (empty($data['mode']) || $data['mode'] == 'reply')
			{
				return;
			}

			$mode = $data['mode'];

			if (!empty($data['post_data']['topic_id']))
			{
				$topic_id = $data['post_data']['topic_id'];
			}

			$is_edit_first_post = $topic_id && $this->is_edit_first_post($mode, $data['post_data']);
			if ($this->is_new_topic($mode) || $is_edit_first_post)
			{
				$page_data = $this->get_template_data_for_topic($topic_id, $is_edit_first_post);
				$data['page_data'] = array_merge($data['page_data'], $page_data);
				$event->set_data($data);
			}
		}
	}

	public function topic_desc_add($event)
	{
		$event['data'] = array_merge($event['data'], array(
			'topic_desc'	=> $this->request->variable('topic_desc', '', true),
		));
	}

	public function submit_post_modify_sql_data($event)
	{
		$mode = $event['post_mode'];
		$data = $event['data'];

		$data_sql = $event['sql_data'];
		if (in_array($mode, array('post', 'edit_topic', 'edit_first_post')))
		{
			$data_sql[TOPICS_TABLE]['sql'][tables::TOPICDESC] = $data['topic_desc'];
		}
		$event['sql_data'] = $data_sql;
	}

	public function topic_desc_add_viewtopic($event)
	{
		$topic_data = $event['topic_data'];
		$this->template->assign_var('TOPIC_DESC',censor_text($topic_data[tables::TOPICDESC]));
	}

	public function modify_message_text($event)
	{
		$event['post_data'] = array_merge($event['post_data'], array(
			'topic_desc'	=> $this->request->variable(tables::TOPICDESC, '', true),
		));
	}

	public function modify_topicrow($event)
	{
		$row = $event['row'];
		if (!empty($row[tables::TOPICDESC]))
		{
			$topic_row = $event['topic_row'];
			$topic_row['TOPIC_DESC'] = censor_text($row[tables::TOPICDESC]);
			$event['topic_row'] = $topic_row;
		}
		
	/**
	 * Event: core.viewforum_modify_topicrow
	 *
	 * Get and assign tags to topic-row-template -> SEO4ALL_TOPICTAGS.
	 *
	 * Note that we assign a string which includes the a-href-links already,
	 * because we cannot assign sub-blocks before the outer-block with
	 * assign_block_vars(...) and the event is before the actual assignment.
	 *
	 * @param $event
	 */
		if ($this->config[prefixes::CONFIG . '_display_tags_in_viewforum'])
		{
			$data = $event->get_data();
			$topic_id = (int) $data['row']['topic_id'];
			$forum_id = (int) $data['row']['forum_id'];

			if ($this->tags_manager->is_tagging_enabled_in_forum($forum_id))
			{
				$tags = $this->tags_manager->get_assigned_tags($topic_id);
				if (!empty($tags))
				{
					// we cannot use assign_block_vars('topicrow.tags', ...) here, because the block 'topicrow' is not yet assigned
					// add links
					$this->assign_tags_to_template('meta_tags_tmp', $tags);
					// small_tag.html might want to use our extension's css.
					$this->template->assign_var('S_SEO4ALL_TAGS_INCLUDE_CSS', true);
					$rendered_tags = $this->template->assign_display('@thetopfew_seo4all/small_tag.html');
					// remove temporary data
					$this->template->destroy_block_vars('meta_tags_tmp');

					// assign the template data
					$data['topic_row']['SEO4ALL_TOPICTAGS'] = $rendered_tags;

					$event->set_data($data);
				}
			}
		}
	}

	public function search_modify_tpl_ary($event)
	{
		$row = $event['row'];
		if ($event['show_results'] == 'topics' && !empty($row[tables::TOPICDESC]))
		{
			$tpl_array = $event['tpl_ary'];
			$tpl_array['TOPIC_DESC'] = censor_text($row[tables::TOPICDESC]);
			$event['tpl_ary'] = $tpl_array;
		}
	}
	
	public function forumlist_description($event)
	{
		$sitename = $this->config['sitename'];
		$site_desc = $this->config['site_desc'];
		
		if (strlen($site_desc) > self::MAX_LENGTH)
		{
			$site_desc = substr($site_desc, 0, self::MAX_LENGTH) . '..';
		}
		
		$this->template->assign_vars(array(       #possibly add later: <meta name="google-site-verification" content="{GOOGLE_AUTH}">
			'META'				=> '<meta name="author" content="' . $sitename . '">
									<meta name="description" content="' . trim($site_desc) . '">',
		));
	}
	
	public function viewforum_description($event)
	{
		$forum_data = $event['forum_data'];
		if (!empty($forum_data['forum_desc']))
		{
			$forum_desc = $this->strip_code($forum_data['forum_desc']);
			if (strlen($forum_desc) > self::MAX_LENGTH)
			{
				$forum_desc = substr($forum_desc, 0, self::MAX_LENGTH) . '..';
			}
			
			$sitename = $this->config['sitename'];
			$this->template->assign_vars(array( 
				'META'				=> '<meta name="author" content="' . $sitename . '">
										<meta name="description" content="' . trim($forum_desc) . '">',
			));
		}
	}

	public function viewtopic_assign_template_vars_before($event)
	{
		$topic_data = $event['topic_data'];

		$t_author = $topic_data['topic_first_poster_name'];
		$user_desc = $this->strip_code($topic_data[tables::TOPICDESC]);
				
		// Assign {META} based on user topic description submit
		if (!empty($user_desc)) 
		{
			// Assign, based on tags enabled or not
			if ($this->tags_manager->is_tagging_enabled_in_forum($topic_data['forum_id'])) 
			{
				$tags = $this->tags_manager->get_assigned_tags($topic_data['topic_id']);
				if (!empty($tags)) 
				{
					$this->assign_tags_to_template('meta_topic_tags', $tags);
					$this->template->assign_vars(array(
						'SHOW_TAGS'			=> true,
						'META'				=> '<meta name="author" content="' . $t_author . '">
												<meta name="description" content="' . $user_desc . '">
												<meta name="keywords" content="' . join(', ', $tags) . '">',
					));
					// tags might want to use our extension's css.
					$this->template->assign_var('S_SEO4ALL_TAGS_INCLUDE_CSS', true);
				} 
				else 
				{
					$this->template->assign_vars(array(
						'META'				=> '<meta name="author" content="' . $t_author . '">
												<meta name="description" content="' . $user_desc . '">',
					));
				}
			}
			else
			{ // if tags are disabled
				$this->template->assign_vars(array(
					'META'				=> '<meta name="author" content="' . $t_author . '">
											<meta name="description" content="' . $user_desc . '">',
				));
			}
		} 
		// Assign {META} based on a auto generated topic description when no user defined
		else 
		{			
			//Fetch our topic's first post_text for any viewtopic=?request
			$post_id = $topic_data['topic_first_post_id'];
			$auto_topic_desc = $this->get_post_text($post_id);
			
			//Strip BBcodes & clean up urls (what about smileys, damn it)
			$auto_topic_desc = $this->strip_code($auto_topic_desc);
			$auto_topic_desc = strip_tags($auto_topic_desc);
			
			//$auto_topic_desc = smiley_text($auto_topic_desc); //doesn't seem to work, yet..
			
			// Limit chars
			if (strlen($auto_topic_desc) >= self::MAX_LENGTH) 
			{
				$auto_topic_desc = substr($auto_topic_desc, 0, self::MAX_LENGTH) . '..';
			}

			// Assign, based on tags enabled or not
			if ($this->tags_manager->is_tagging_enabled_in_forum($topic_data['forum_id'])) 
			{
				$tags = $this->tags_manager->get_assigned_tags($topic_data['topic_id']);
								
				if (!empty($tags))
				{
					$this->assign_tags_to_template('meta_topic_tags', $tags);
					$this->template->assign_vars(array( //add later: <meta name="google-site-verification" content="{GOOGLE_AUTH}">
						'SHOW_TAGS'			=> true,
						'META'				=> '<meta name="author" content="' . $t_author . '">
												<meta name="description" content="' . $auto_topic_desc . '">
												<meta name="keywords" content="' . join(', ', $tags) . '">',
					));
					// tags might want to use our extension's css.
					$this->template->assign_var('S_SEO4ALL_TAGS_INCLUDE_CSS', true);
				} 
				else 
				{
					$this->template->assign_vars(array(
						'META'				=> '<meta name="author" content="' . $t_author . '">
												<meta name="description" content="' . $auto_topic_desc . '">',
					));
				}
			}
			else
			{ //if tags are disabled
				$this->template->assign_vars(array(
					'META'				=> '<meta name="author" content="' . $t_author . '">
											<meta name="description" content="' . $auto_topic_desc . '">',
				));
			}
		}
		
		$event['post_id'] = $post_id;
		$event['topic_data'] = $topic_data;
	}
	
	private function get_post_text($post_id)
	{
		global $db;
		$sql = 'SELECT post_text
			FROM ' . POSTS_TABLE . ' 
			WHERE post_id = ' . $post_id .' ';
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		$post_text = $row['post_text'];
		
		return $post_text;
	}
	
	private function strip_code($text)
	{
		$text = censor_text($text); // Run thru ACP->Posting->Word Censoring blacklist
		strip_bbcode($text); // Bu-bye all bbcodes!

		// Clean up any left over http urls
		$text = preg_replace('/\b(https?):\/\/[-A-Z0-9+&@#\/%?=~_|$!:,.;]*[A-Z0-9+&@#\/%=~_|$]/i', '', $text);
			
			// Clean up any types out urls with only, www.links.($end)
			$start = '\www.';
			// THANK YOU ALESSIO! End domain list by: http://www.axew3.com/www/data-hints/w3-all-top-level-domains-names-array.php
			$end = '\.aaa|.aarp|.abarth|.abb|.abbott|.abbvie|.abc|.able|.abogado|.abudhabi|.ac|.academy|.accenture|.accountant|.accountants|.aco|.active|.actor|.ad|.adac|.ads|.adult|.ae|.aeg|.aero|.aetna|.af|.afamilycompany|.afl|.africa|.ag|.agakhan|.agency|.ai|.aig|.aigo|.airbus|.airforce|.airtel|.akdn|.al|.alfaromeo|.alibaba|.alipay|.allfinanz|.allstate|.ally|.alsace|.alstom|.am|.americanexpress|.americanfamily|.amex|.amfam|.amica|.amsterdam|.analytics|.android|.anquan|.anz|.ao|.aol|.apartments|.app|.apple|.aq|.aquarelle|.ar|.arab|.aramco|.archi|.army|.arpa|.art|.arte|.as|.asda|.asia|.associates|.at|.athleta|.attorney|.au|.auction|.audi|.audible|.audio|.auspost|.author|.auto|.autos|.avianca|.aw|.aws|.ax|.axa|.az|.azure|.ba|.baby|.baidu|.banamex|.bananarepublic|.band|.bank|.bar|.barcelona|.barclaycard|.barclays|.barefoot|.bargains|.baseball|.basketball|.bauhaus|.bayern|.bb|.bbc|.bbt|.bbva|.bcg|.bcn|.bd|.be|.beats|.beauty|.beer|.bentley|.berlin|.best|.bestbuy|.bet|.bf|.bg|.bh|.bharti|.bi|.bible|.bid|.bike|.bing|.bingo|.bio|.biz|.bj|.black|.blackfriday|.blanco|.blockbuster|.blog|.bloomberg|.blue|.bm|.bms|.bmw|.bn|.bnl|.bnpparibas|.bo|.boats|.boehringer|.bofa|.bom|.bond|.boo|.book|.booking|.boots|.bosch|.bostik|.boston|.bot|.boutique|.box|.br|.bradesco|.bridgestone|.broadway|.broker|.brother|.brussels|.bs|.bt|.budapest|.bugatti|.build|.builders|.business|.buy|.buzz|.bv|.bw|.by|.bz|.bzh|.ca|.cab|.cafe|.cal|.call|.calvinklein|.cam|.camera|.camp|.cancerresearch|.canon|.capetown|.capital|.capitalone|.car|.caravan|.cards|.care|.career|.careers|.cars|.cartier|.casa|.case|.caseih|.cash|.casino|.cat|.catering|.catholic|.cba|.cbn|.cbre|.cbs|.cc|.cd|.ceb|.center|.ceo|.cern|.cf|.cfa|.cfd|.cg|.ch|.chanel|.channel|.chase|.chat|.cheap|.chintai|.chloe|.christmas|.chrome|.chrysler|.church|.ci|.cipriani|.circle|.cisco|.citadel|.citi|.citic|.city|.cityeats|.ck|.cl|.claims|.cleaning|.click|.clinic|.clinique|.clothing|.cloud|.club|.clubmed|.cm|.cn|.co|.coach|.codes|.coffee|.college|.cologne|.com|.comcast|.commbank|.community|.company|.compare|.computer|.comsec|.condos|.construction|.consulting|.contact|.contractors|.cooking|.cookingchannel|.cool|.coop|.corsica|.country|.coupon|.coupons|.courses|.cr|.credit|.creditcard|.creditunion|.cricket|.crown|.crs|.cruise|.cruises|.csc|.cu|.cuisinella|.cv|.cw|.cx|.cy|.cymru|.cyou|.cz|.dabur|.dad|.dance|.data|.date|.dating|.datsun|.day|.dclk|.dds|.de|.deal|.dealer|.deals|.degree|.delivery|.dell|.deloitte|.delta|.democrat|.dental|.dentist|.desi|.design|.dev|.dhl|.diamonds|.diet|.digital|.direct|.directory|.discount|.discover|.dish|.diy|.dj|.dk|.dm|.dnp|.do|.docs|.doctor|.dodge|.dog|.doha|.domains|.dot|.download|.drive|.dtv|.dubai|.duck|.dunlop|.duns|.dupont|.durban|.dvag|.dvr|.dz|.earth|.eat|.ec|.eco|.edeka|.edu|.education|.ee|.eg|.email|.emerck|.energy|.engineer|.engineering|.enterprises|.epost|.epson|.equipment|.er|.ericsson|.erni|.es|.esq|.estate|.esurance|.et|.etisalat|.eu|.eurovision|.eus|.events|.everbank|.exchange|.expert|.exposed|.express|.extraspace|.fage|.fail|.fairwinds|.faith|.family|.fan|.fans|.farm|.farmers|.fashion|.fast|.fedex|.feedback|.ferrari|.ferrero|.fi|.fiat|.fidelity|.fido|.film|.final|.finance|.financial|.fire|.firestone|.firmdale|.fish|.fishing|.fit|.fitness|.fj|.fk|.flickr|.flights|.flir|.florist|.flowers|.fly|.fm|.fo|.foo|.food|.foodnetwork|.football|.ford|.forex|.forsale|.forum|.foundation|.fox|.fr|.free|.fresenius|.frl|.frogans|.frontdoor|.frontier|.ftr|.fujitsu|.fujixerox|.fun|.fund|.furniture|.futbol|.fyi|.ga|.gal|.gallery|.gallo|.gallup|.game|.games|.gap|.garden|.gb|.gbiz|.gd|.gdn|.ge|.gea|.gent|.genting|.george|.gf|.gg|.ggee|.gh|.gi|.gift|.gifts|.gives|.giving|.gl|.glade|.glass|.gle|.global|.globo|.gm|.gmail|.gmbh|.gmo|.gmx|.gn|.godaddy|.gold|.goldpoint|.golf|.goo|.goodhands|.goodyear|.goog|.google|.gop|.got|.gov|.gp|.gq|.gr|.grainger|.graphics|.gratis|.green|.gripe|.grocery|.group|.gs|.gt|.gu|.guardian|.gucci|.guge|.guide|.guitars|.guru|.gw|.gy|.hair|.hamburg|.hangout|.haus|.hbo|.hdfc|.hdfcbank|.health|.healthcare|.help|.helsinki|.here|.hermes|.hgtv|.hiphop|.hisamitsu|.hitachi|.hiv|.hk|.hkt|.hm|.hn|.hockey|.holdings|.holiday|.homedepot|.homegoods|.homes|.homesense|.honda|.honeywell|.horse|.hospital|.host|.hosting|.hot|.hoteles|.hotels|.hotmail|.house|.how|.hr|.hsbc|.ht|.htc|.hu|.hughes|.hyatt|.hyundai|.ibm|.icbc|.ice|.icu|.id|.ie|.ieee|.ifm|.ikano|.il|.im|.imamat|.imdb|.immo|.immobilien|.in|.industries|.infiniti|.info|.ing|.ink|.institute|.insurance|.insure|.int|.intel|.international|.intuit|.investments|.io|.ipiranga|.iq|.ir|.irish|.is|.iselect|.ismaili|.ist|.istanbul|.it|.itau|.itv|.iveco|.iwc|.jaguar|.java|.jcb|.jcp|.je|.jeep|.jetzt|.jewelry|.jio|.jlc|.jll|.jm|.jmp|.jnj|.jo|.jobs|.joburg|.jot|.joy|.jp|.jpmorgan|.jprs|.juegos|.juniper|.kaufen|.kddi|.ke|.kerryhotels|.kerrylogistics|.kerryproperties|.kfh|.kg|.kh|.ki|.kia|.kim|.kinder|.kindle|.kitchen|.kiwi|.km|.kn|.koeln|.komatsu|.kosher|.kp|.kpmg|.kpn|.kr|.krd|.kred|.kuokgroup|.kw|.ky|.kyoto|.kz|.la|.lacaixa|.ladbrokes|.lamborghini|.lamer|.lancaster|.lancia|.lancome|.land|.landrover|.lanxess|.lasalle|.lat|.latino|.latrobe|.law|.lawyer|.lb|.lc|.lds|.lease|.leclerc|.lefrak|.legal|.lego|.lexus|.lgbt|.li|.liaison|.lidl|.life|.lifeinsurance|.lifestyle|.lighting|.like|.lilly|.limited|.limo|.lincoln|.linde|.link|.lipsy|.live|.living|.lixil|.lk|.loan|.loans|.locker|.locus|.loft|.lol|.london|.lotte|.lotto|.love|.lpl|.lplfinancial|.lr|.ls|.lt|.ltd|.ltda|.lu|.lundbeck|.lupin|.luxe|.luxury|.lv|.ly|.ma|.macys|.madrid|.maif|.maison|.makeup|.man|.management|.mango|.map|.market|.marketing|.markets|.marriott|.marshalls|.maserati|.mattel|.mba|.mc|.mcd|.mcdonalds|.mckinsey|.md|.me|.med|.media|.meet|.melbourne|.meme|.memorial|.men|.menu|.meo|.merckmsd|.metlife|.mg|.mh|.miami|.microsoft|.mil|.mini|.mint|.mit|.mitsubishi|.mk|.ml|.mlb|.mls|.mm|.mma|.mn|.mo|.mobi|.mobile|.mobily|.moda|.moe|.moi|.mom|.monash|.money|.monster|.montblanc|.mopar|.mormon|.mortgage|.moscow|.moto|.motorcycles|.mov|.movie|.movistar|.mp|.mq|.mr|.ms|.msd|.mt|.mtn|.mtr|.mu|.museum|.mutual|.mv|.mw|.mx|.my|.mz|.na|.nab|.nadex|.nagoya|.name|.nationwide|.natura|.navy|.nba|.nc|.ne|.nec|.net|.netbank|.netflix|.network|.neustar|.new|.newholland|.news|.next|.nextdirect|.nexus|.nf|.nfl|.ng|.ngo|.nhk|.ni|.nico|.nike|.nikon|.ninja|.nissan|.nissay|.nl|.no|.nokia|.northwesternmutual|.norton|.now|.nowruz|.nowtv|.np|.nr|.nra|.nrw|.ntt|.nu|.nyc|.nz|.obi|.observer|.off|.office|.okinawa|.olayan|.olayangroup|.oldnavy|.ollo|.om|.omega|.one|.ong|.onl|.online|.onyourside|.ooo|.open|.oracle|.orange|.org|.organic|.origins|.osaka|.otsuka|.ott|.ovh|.pa|.page|.pamperedchef|.panasonic|.panerai|.paris|.pars|.partners|.parts|.party|.passagens|.pay|.pccw|.pe|.pet|.pf|.pfizer|.pg|.ph|.pharmacy|.phd|.philips|.phone|.photo|.photography|.photos|.physio|.piaget|.pics|.pictet|.pictures|.pid|.pin|.ping|.pink|.pioneer|.pizza|.pk|.pl|.place|.play|.playstation|.plumbing|.plus|.pm|.pn|.pnc|.pohl|.poker|.politie|.porn|.post|.pr|.pramerica|.praxi|.press|.prime|.pro|.prod|.productions|.prof|.progressive|.promo|.properties|.property|.protection|.pru|.prudential|.ps|.pt|.pub|.pw|.pwc|.py|.qa|.qpon|.quebec|.quest|.qvc|.racing|.radio|.raid|.re|.read|.realestate|.realtor|.realty|.recipes|.red|.redstone|.redumbrella|.rehab|.reise|.reisen|.reit|.reliance|.ren|.rent|.rentals|.repair|.report|.republican|.rest|.restaurant|.review|.reviews|.rexroth|.rich|.richardli|.ricoh|.rightathome|.ril|.rio|.rip|.rmit|.ro|.rocher|.rocks|.rodeo|.rogers|.room|.rs|.rsvp|.ru|.rugby|.ruhr|.run|.rw|.rwe|.ryukyu|.sa|.saarland|.safe|.safety|.sakura|.sale|.salon|.samsclub|.samsung|.sandvik|.sandvikcoromant|.sanofi|.sap|.sapo|.sarl|.sas|.save|.saxo|.sb|.sbi|.sbs|.sc|.sca|.scb|.schaeffler|.schmidt|.scholarships|.school|.schule|.schwarz|.science|.scjohnson|.scor|.scot|.sd|.se|.search|.seat|.secure|.security|.seek|.select|.sener|.services|.ses|.seven|.sew|.sex|.sexy|.sfr|.sg|.sh|.shangrila|.sharp|.shaw|.shell|.shia|.shiksha|.shoes|.shop|.shopping|.shouji|.show|.showtime|.shriram|.si|.silk|.sina|.singles|.site|.sj|.sk|.ski|.skin|.sky|.skype|.sl|.sling|.sm|.smart|.smile|.sn|.sncf|.so|.soccer|.social|.softbank|.software|.sohu|.solar|.solutions|.song|.sony|.soy|.space|.spiegel|.spot|.spreadbetting|.sr|.srl|.srt|.st|.stada|.staples|.star|.starhub|.statebank|.statefarm|.statoil|.stc|.stcgroup|.stockholm|.storage|.store|.stream|.studio|.study|.style|.su|.sucks|.supplies|.supply|.support|.surf|.surgery|.suzuki|.sv|.swatch|.swiftcover|.swiss|.sx|.sy|.sydney|.symantec|.systems|.sz|.tab|.taipei|.talk|.taobao|.target|.tatamotors|.tatar|.tattoo|.tax|.taxi|.tc|.tci|.td|.tdk|.team|.tech|.technology|.tel|.telecity|.telefonica|.temasek|.tennis|.teva|.tf|.tg|.th|.thd|.theater|.theatre|.tiaa|.tickets|.tienda|.tiffany|.tips|.tires|.tirol|.tj|.tjmaxx|.tjx|.tk|.tkmaxx|.tl|.tm|.tmall|.tn|.to|.today|.tokyo|.tools|.top|.toray|.toshiba|.total|.tours|.town|.toyota|.toys|.tr|.trade|.trading|.training|.travel|.travelchannel|.travelers|.travelersinsurance|.trust|.trv|.tt|.tube|.tui|.tunes|.tushu|.tv|.tvs|.tw|.tz|.ua|.ubank|.ubs|.uconnect|.ug|.uk|.unicom|.university|.uno|.uol|.ups|.us|.uy|.uz|.va|.vacations|.vana|.vanguard|.vc|.ve|.vegas|.ventures|.verisign|.versicherung|.vet|.vg|.vi|.viajes|.video|.vig|.viking|.villas|.vin|.vip|.virgin|.visa|.vision|.vista|.vistaprint|.viva|.vivo|.vlaanderen|.vn|.vodka|.volkswagen|.volvo|.vote|.voting|.voto|.voyage|.vu|.vuelos|.wales|.walmart|.walter|.wang|.wanggou|.warman|.watch|.watches|.weather|.weatherchannel|.webcam|.weber|.website|.wed|.wedding|.weibo|.weir|.wf|.whoswho|.wien|.wiki|.williamhill|.win|.windows|.wine|.winners|.wme|.wolterskluwer|.woodside|.work|.works|.world|.wow|.ws|.wtc|.wtf|.xbox|.xerox|.xfinity|.xihuan|.xin|.xn--11b4c3d|.xn--1ck2e1b|.xn--1qqw23a|.xn--2scrj9c|.xn--30rr7y|.xn--3bst00m|.xn--3ds443g|.xn--3e0b707e|.xn--3hcrj9c|.xn--3oq18vl8pn36a|.xn--3pxu8k|.xn--42c2d9a|.xn--45br5cyl|.xn--45brj9c|.xn--45q11c|.xn--4gbrim|.xn--54b7fta0cc|.xn--55qw42g|.xn--55qx5d|.xn--5su34j936bgsg|.xn--5tzm5g|.xn--6frz82g|.xn--6qq986b3xl|.xn--80adxhks|.xn--80ao21a|.xn--80aqecdr1a|.xn--80asehdb|.xn--80aswg|.xn--8y0a063a|.xn--90a3ac|.xn--90ae|.xn--90ais|.xn--9dbq2a|.xn--9et52u|.xn--9krt00a|.xn--b4w605ferd|.xn--bck1b9a5dre4c|.xn--c1avg|.xn--c2br7g|.xn--cck2b3b|.xn--cg4bki|.xn--clchc0ea0b2g2a9gcd|.xn--czr694b|.xn--czrs0t|.xn--czru2d|.xn--d1acj3b|.xn--d1alf|.xn--e1a4c|.xn--eckvdtc9d|.xn--efvy88h|.xn--estv75g|.xn--fct429k|.xn--fhbei|.xn--fiq228c5hs|.xn--fiq64b|.xn--fiqs8s|.xn--fiqz9s|.xn--fjq720a|.xn--flw351e|.xn--fpcrj9c3d|.xn--fzc2c9e2c|.xn--fzys8d69uvgm|.xn--g2xx48c|.xn--gckr3f0f|.xn--gecrj9c|.xn--gk3at1e|.xn--h2breg3eve|.xn--h2brj9c|.xn--h2brj9c8c|.xn--hxt814e|.xn--i1b6b1a6a2e|.xn--imr513n|.xn--io0a7i|.xn--j1aef|.xn--j1amh|.xn--j6w193g|.xn--jlq61u9w7b|.xn--jvr189m|.xn--kcrx77d1x4a|.xn--kprw13d|.xn--kpry57d|.xn--kpu716f|.xn--kput3i|.xn--l1acc|.xn--lgbbat1ad8j|.xn--mgb9awbf|.xn--mgba3a3ejt|.xn--mgba3a4f16a|.xn--mgba7c0bbn0a|.xn--mgbaakc7dvf|.xn--mgbaam7a8h|.xn--mgbab2bd|.xn--mgbai9azgqp6j|.xn--mgbayh7gpa|.xn--mgbb9fbpob|.xn--mgbbh1a|.xn--mgbbh1a71e|.xn--mgbc0a9azcg|.xn--mgbca7dzdo|.xn--mgberp4a5d4ar|.xn--mgbgu82a|.xn--mgbi4ecexp|.xn--mgbpl2fh|.xn--mgbt3dhd|.xn--mgbtx2b|.xn--mgbx4cd0ab|.xn--mix891f|.xn--mk1bu44c|.xn--mxtq1m|.xn--ngbc5azd|.xn--ngbe9e0a|.xn--ngbrx|.xn--node|.xn--nqv7f|.xn--nqv7fs00ema|.xn--nyqy26a|.xn--o3cw4h|.xn--ogbpf8fl|.xn--p1acf|.xn--p1ai|.xn--pbt977c|.xn--pgbs0dh|.xn--pssy2u|.xn--q9jyb4c|.xn--qcka1pmc|.xn--qxam|.xn--rhqv96g|.xn--rovu88b|.xn--rvc1e0am3e|.xn--s9brj9c|.xn--ses554g|.xn--t60b56a|.xn--tckwe|.xn--tiq49xqyj|.xn--unup4y|.xn--vermgensberater-ctb|.xn--vermgensberatung-pwb|.xn--vhquv|.xn--vuq861b|.xn--w4r85el8fhu5dnra|.xn--w4rs40l|.xn--wgbh1c|.xn--wgbl6a|.xn--xhq521b|.xn--xkc2al3hye2a|.xn--xkc2dl3a5ee0h|.xn--y9a3aq|.xn--yfro4i67o|.xn--ygbi2ammx|.xn--zfr164b|.xperia|.xxx|.xyz|.yachts|.yahoo|.yamaxun|.yandex|.ye|.yodobashi|.yoga|.yokohama|.you|.youtube|.yt|.yun|.za|.zappos|.zara|.zero|.zip|.zippo|.zm|.zone|.zuerich|.zw';
			
			$text = preg_replace('#('.$start.')(.*.)('.$end.')#', '', $text); // Runs through all, finally!
		
		// Clean up quotes, spaces, returns.
		$text = str_replace(array("&quot;", "\n", "\t", "\r"), '', $text);
		
		return trim($text);
	}

	//TOPIC TAGS
	/**
	 * Reads all tags from request variable 'seo_topictags' and splits them by the separator (default: comma (',')) and trims them.
	 * NOTE: These tags might be dirty!
	 *
	 * @return array of dirty tags
	 */
	private function get_tags_from_post_request()
	{
		$tags_string = utf8_normalize_nfc($this->request->variable('seo_topictags', '', true));
		$tags_string = rawurldecode(base64_decode($tags_string));

		if ('' === $tags_string)
		{
			return array();
		}

		$tagsJson = json_decode($tags_string, true);
		$tags = array();

		for ($i = 0, $count = sizeof($tagsJson); $i<$count; $i++)
		{
			$tags[] = trim($tagsJson[$i]['text']);
		}

		return $tags;
	}

	/**
	 * Event: core.index_modify_page_title
	 */
	public function index_modify_page_title($event)
	{
		if ($this->config[prefixes::CONFIG . '_display_tagcloud_on_index'])
		{
			$this->template->assign_vars(array(
				'SEO4ALL_DISPLAY_TAGCLOUD_ON_INDEX'	=> true,
				'SEO4ALL_TAGCLOUD_LINK'				=> $this->helper->route('thetopfew_seo4all_tags_controller'),
			));
			$this->tagcloud_manager->assign_tagcloud_to_template();
		}
	}

	/**
	 * Event: core.modify_posting_parameters
	 *
	 * Validate the tags and create an error if any tag is invalid.
	 */
	public function modify_posting_parameters($event)
	{
		if ($this->auth->acl_gets(permissions::USE_TAGS, permissions::ADMIN_EDIT_TAGS, permissions::MOD_EDIT_TAGS))
		{
			$data = $event->get_data();
			$tags = $this->get_tags_from_post_request();

			$all_tags = $this->tags_manager->split_valid_tags($tags);
			$invalid_tags = $all_tags['invalid'];

			if (sizeof($invalid_tags))
			{
				$this->user->add_lang_ext('thetopfew/seo4all', 'common'); //required to display error client-side
				$data['error'][] = $this->user->lang('SEO4ALL_TAGS_INVALID', join(', ', $invalid_tags));
			}

			$event->set_data($data);
		}
	}

	/**
	 * Checks whether the mode indicates a new topic or not.
	 * @param string $mode the mode.
	 * @return true if mode == post (indicating a new topic), false otherwise
	 */
	private function is_new_topic($mode)
	{
		return $is_new_topic = $mode == 'post';
	}

	/**
	 * Check whether the post data indicates that the first post of a topic is edited or not.
	 *
	 * @param string $mode the events data mode
	 * @param array $post_data the event data
	 * @return boolean true if it is a first post edit, false otherwise
	 */
	private function is_edit_first_post($mode, array $post_data)
	{
		$post_id = $topic_first_post_id = false;
		if (!empty($post_data['topic_first_post_id']))
		{
			$topic_first_post_id = $post_data['topic_first_post_id'];
		}
		if (!empty($post_data['post_id']))
		{
			$post_id = $post_data['post_id'];
		}
		return $mode == 'edit' && $post_id && $post_id == $topic_first_post_id;
	}

	/**
	 * Calculates the template data for the topic
	 *
	 * @param int $topic_id the id of the topic
	 * @param boolean $is_edit_first_post whether it is a first post edit or not
	 * @return array the page data
	 */
	private function get_template_data_for_topic($topic_id, $is_edit_first_post)
	{
		$page_data = array();
		$page_data['SEO4ALL_TAGS_SHOW_FIELD'] = true;

		// do we got some preview-data?
		$tags = array();
		if ($this->request->is_set_post('seo_topictags')) //form key?
		{
			// use data from post-request
			$tags = $this->get_tags_from_post_request();
		}
		else if ($is_edit_first_post)
		{
			// use data from db
			$tags = $this->tags_manager->get_assigned_tags($topic_id);
		}

		$page_data['SEO4ALL_TAGS'] = base64_encode(rawurlencode(json_encode($tags)));

		$page_data['SEO4ALL_ALLOWED_TAGS_REGEX'] = $this->config[prefixes::CONFIG . '_allowed_tags_regex'];
		$page_data['SEO4ALL_TAGS_CONVERT_SPACE_TO_MINUS'] = $this->config[prefixes::CONFIG . '_convert_space_to_minus'] ? 'true' : 'false';

		$page_data['S_SEO4ALL_WHITELIST_ENABLED'] = $this->config[prefixes::CONFIG . '_whitelist_enabled'];

		if ($this->config[prefixes::CONFIG . '_whitelist_enabled'])
		{
			$page_data['S_SEO4ALL_WHITELIST_ENABLED'] = true;
			$tags = $this->tags_manager->get_whitelist_tags();
			for ($i = 0, $size = sizeof($tags); $i < $size; $i++)
			{
				$this->template->assign_block_vars('seo4all_tags_whitelist', array(
					'LINK' => '#',
					'NAME' => $tags[$i],
				));
			}
		}
		else
		{
			$page_data['SEO4ALL_ALLOWED_TAGS_EXP'] = $this->config[prefixes::CONFIG . '_allowed_tags_exp_for_users'];
		}
		$page_data['S_SEO4ALL_INCLUDE_NG_TAGS_INPUT'] = true;
		$page_data['S_SEO4ALL_TAGS_INCLUDE_CSS'] = true;
		return $page_data;
	}

	/**
	 * Assigns the given tags to the template block
	 *
	 * @param string $block_name the name of the template block
	 * @param array $tags the tags to assign
	 */
	private function assign_tags_to_template($block_name, array $tags)
	{
		foreach ($tags as $tag)
		{
			$this->template->assign_block_vars($block_name, array (
				'NAME'	=> $tag,
				'LINK'	=> $this->helper->route('thetopfew_seo4all_show_tag_controller', array(
				'tags'	=> urlencode($tag),
				)),
			));
		}
	}
	
	/**
	 * Event: core.delete_topics_before_query
	 *
	 * prune tags when topic is deleted
	 *
	 * @param $event
	 */
	public function delete_topics_before_query($event)
	{
		$data = $event->get_data();
		$topic_ids = $data['topic_ids'];
		$this->tags_manager->remove_all_tags_from_topics($topic_ids, true);
	}

	
	//SITEMAP
	/**
	* Update post modification time when posting or edit
	*
	* @return null
	* @access public
	*/
	public function update_post_modified_time($event)
	{
		$data = $event['data'];

		$sql = 'UPDATE ' . POSTS_TABLE . '
			SET post_modified = ' . (int) time() . '
			WHERE post_id = ' . (int) $data['post_id'];
		$this->db->sql_query($sql);
		
		
	/**
	 * Event: core.postingsubmit_post_end
	 *
	 * After a posting we assign the tags to the topic
	 */
		if ($this->auth->acl_gets(permissions::USE_TAGS, permissions::ADMIN_EDIT_TAGS, permissions::MOD_EDIT_TAGS))
		{
			$event_data = $event->get_data();
			$data = $event_data['data'];
			$mode = $event_data['mode'];
			if ($this->is_new_topic($mode) || $this->is_edit_first_post($mode, $data))
			{
				$tags = $this->get_tags_from_post_request();
				$all_tags = $this->tags_manager->split_valid_tags($tags);
				$valid_tags = $all_tags['valid'];
				$this->tags_manager->assign_tags_to_topic($data['topic_id'], $valid_tags);
			}
		}
	}

	public function seo_return($event)
	{
		if (!defined('PHPBB_WORK_INFO'))
		{
			$this->template->assign_vars(array(
				'PHPBB_WORK_SITEMAP'	=> ($this->config['default_lang'] == 'en') ? true : false,
			));

			define('PHPBB_WORK_INFO', 1);
		}
	}

}