<?php
/**
* SEO4ALL. Extension for the phpBB Forum Software package.
*
* @copyright (c) 2017 https://thetopfew.com
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
**/

namespace thetopfew\seo4all\migrations;

use thetopfew\seo4all\service\constants\prefixes;
use thetopfew\seo4all\service\constants\tables;

class v2_0_0 extends \phpbb\db\migration\migration
{
	public function effectively_installed()
	{
		return isset($this->config['seo4all_version']) && version_compare($this->config['seo4all_version'], '2.0.0', '>=');
	}

	public static function depends_on()
	{
		return array('\thetopfew\seo4all\migrations\v_init',);
	}
	
	public function update_data()
	{
		// convert whitelist to json
		$whitelist = $this->config[prefixes::CONFIG.'_whitelist'];
		if (empty($whitelist))
		{
			$whitelist = array();
		}
		else
		{
			$whitelist = explode(',', $whitelist);
		}
		$whitelist = json_encode($whitelist);

		// convert blacklist to json
		$blacklist = $this->config[prefixes::CONFIG.'_blacklist'];
		if (empty($blacklist))
		{
			$blacklist = array();
		}
		else
		{
			$blacklist = explode(',', $blacklist);
		}
		$blacklist = json_encode($blacklist);
		
		return array(
			// update version
			array('config.update', array('seo4all_version', '2.0.0')),

			// put white- and blacklist from config to config_text	
			array('custom', array(array($this, 'calculate_lowercase_tags'))),
			array('config.remove', array(prefixes::CONFIG.'_whitelist')),
			array('config.remove', array(prefixes::CONFIG.'_blacklist')),
			array('config_text.add', array(prefixes::CONFIG.'_whitelist', $whitelist)),
			array('config_text.add', array(prefixes::CONFIG.'_blacklist', $blacklist)),
		);
	}
	
	public function calculate_lowercase_tags()
	{
		$sql = 'SELECT id, tag
			FROM ' . $this->table_prefix . tables::TAGS;
		$result = $this->db->sql_query($sql);

		while ($row = $this->db->sql_fetchrow($result))
		{
			$sql_ary = array(
				'tag_lowercase' =>utf8_strtolower($row['tag']),
			);
			$sql = 'UPDATE ' . $this->table_prefix . tables::TAGS . '
				SET ' . $this->db->sql_build_array('UPDATE', $sql_ary) . '
				WHERE id = ' . $row['id'];
			$this->db->sql_query($sql);
		}

		$this->db->sql_freeresult($result);
	}
}
